@@ -14,9 +14,11 @@ This includes:
 from __future__ import print_function
 
 import os
+import re
+import pipes
+import shutil
 import sys
 
-import pipes
 from distutils import log
 from distutils.cmd import Command
 from fnmatch import fnmatch
@@ -68,6 +70,11 @@ execfile(pjoin(repo_root, name, '_version.py'), version_ns)
 version = version_ns['__version__']
 
 
+# vendored from pep440 package, we allow `.dev` suffix without trailing number.
+loose_pep440re = re.compile(r'^([1-9]\d*!)?(0|[1-9]\d*)(\.(0|[1-9]\d*))*((a|b|rc)(0|[1-9]\d*))?(\.post(0|[1-9]\d*))?(\.dev(0|[1-9]\d*)?)?$')
+if not loose_pep440re.match(version):
+    raise ValueError('Invalid version number `%s`, please follow pep440 convention or pip will get confused about which package is more recent.' % version)
+
 #---------------------------------------------------------------------------
 # Find packages
 #---------------------------------------------------------------------------
@@ -101,6 +108,7 @@ def find_package_data():
     excludes = [
         pjoin('static', 'components'),
         pjoin('static', '*', 'less'),
+        pjoin('static', '*', 'node_modules')
     ]
 
     # walk notebook resources:
@@ -128,9 +136,11 @@ def find_package_data():
         pjoin(components, "bootstrap", "js", "bootstrap.min.js"),
         pjoin(components, "bootstrap-tour", "build", "css", "bootstrap-tour.min.css"),
         pjoin(components, "bootstrap-tour", "build", "js", "bootstrap-tour.min.js"),
+        pjoin(components, "font-awesome", "css", "*.css"),
         pjoin(components, "es6-promise", "*.js"),
         pjoin(components, "font-awesome", "fonts", "*.*"),
         pjoin(components, "google-caja", "html-css-sanitizer-minified.js"),
+        pjoin(components, "jed", "jed.js"),
         pjoin(components, "jquery", "jquery.min.js"),
         pjoin(components, "jquery-typeahead", "dist", "jquery.typeahead.min.js"),
         pjoin(components, "jquery-typeahead", "dist", "jquery.typeahead.min.css"),
@@ -138,11 +148,17 @@ def find_package_data():
         pjoin(components, "jquery-ui", "themes", "smoothness", "jquery-ui.min.css"),
         pjoin(components, "jquery-ui", "themes", "smoothness", "images", "*"),
         pjoin(components, "marked", "lib", "marked.js"),
+        pjoin(components, "preact", "index.js"),
+        pjoin(components, "preact-compat", "index.js"),
+        pjoin(components, "proptypes", "index.js"),
         pjoin(components, "requirejs", "require.js"),
+        pjoin(components, "requirejs-plugins", "src", "json.js"),
+        pjoin(components, "requirejs-text", "text.js"),
         pjoin(components, "underscore", "underscore-min.js"),
         pjoin(components, "moment", "moment.js"),
-        pjoin(components, "moment", "min", "moment.min.js"),
-        pjoin(components, "term.js", "src", "term.js"),
+        pjoin(components, "moment", "min", "*.js"),
+        pjoin(components, "xterm.js", "dist", "xterm.js"),
+        pjoin(components, "xterm.js", "dist", "xterm.css"),
         pjoin(components, "text-encoding", "lib", "encoding.js"),
     ])
 
@@ -156,7 +172,7 @@ def find_package_data():
     mj = lambda *path: pjoin(components, 'MathJax', *path)
     static_data.extend([
         mj('MathJax.js'),
-        mj('config', 'TeX-AMS_HTML-full.js'),
+        mj('config', 'TeX-AMS-MML_HTMLorMML-full.js'),
         mj('config', 'Safe.js'),
     ])
     
@@ -178,6 +194,7 @@ def find_package_data():
         mj('jax', 'input', 'TeX'),
         mj('jax', 'output', 'HTML-CSS', 'fonts', 'STIX-Web'),
         mj('jax', 'output', 'SVG', 'fonts', 'STIX-Web'),
+        mj('jax', 'element', 'mml'),
     ]:
         for parent, dirs, files in os.walk(tree):
             for f in files:
@@ -191,6 +208,8 @@ def find_package_data():
     package_data = {
         'notebook' : ['templates/*'] + static_data,
         'notebook.tests' : js_tests,
+        'notebook.bundler.tests': ['resources/*', 'resources/*/*', 'resources/*/*/.*'],
+        'notebook.services.api': ['api.yaml'],
     }
     
     return package_data
@@ -338,6 +357,7 @@ class Bower(Command):
             return True
         if not os.path.exists(self.bower_dir):
             return True
+        
         return mtime(self.bower_dir) < mtime(pjoin(repo_root, 'bower.json'))
 
     def should_run_npm(self):
@@ -347,7 +367,7 @@ class Bower(Command):
         if not os.path.exists(self.node_modules):
             return True
         return mtime(self.node_modules) < mtime(pjoin(repo_root, 'package.json'))
-    
+
     def run(self):
         if not self.should_run():
             print("bower dependencies up to date")
@@ -371,6 +391,7 @@ class Bower(Command):
             print("Failed to run bower: %s" % e, file=sys.stderr)
             print("You can install js dependencies with `npm install`", file=sys.stderr)
             raise
+        # self.npm_components()
         os.utime(self.bower_dir, None)
         # update package data in case this created new files
         update_package_data(self.distribution)
@@ -440,6 +461,7 @@ class CompileJS(Command):
     
     def sources(self, name):
         """Generator yielding .js sources that an application depends on"""
+        yield pjoin(repo_root, 'tools', 'build-main.js')
         yield pjoin(static, name, 'js', 'main.js')
 
         for sec in [name, 'base', 'auth']:
@@ -536,8 +558,8 @@ def css_js_prerelease(command, strict=False):
                 return
 
             try:
-                self.distribution.run_command('css')
                 self.distribution.run_command('js')
+                self.distribution.run_command('css')
             except Exception as e:
                 # refresh missing
                 missing = [ t for t in targets if not os.path.exists(t) ]
